@@ -1,10 +1,9 @@
 """Utilities related to importing modules and symbols by name."""
-import importlib
 import os
 import sys
 import warnings
 from contextlib import contextmanager
-from importlib import reload
+from importlib import import_module, reload
 
 try:
     from importlib.metadata import entry_points
@@ -52,8 +51,13 @@ def instantiate(name, *args, **kwargs):
 @contextmanager
 def cwd_in_path():
     """Context adding the current working directory to sys.path."""
-    cwd = os.getcwd()
-    if cwd in sys.path:
+    try:
+        cwd = os.getcwd()
+    except FileNotFoundError:
+        cwd = None
+    if not cwd:
+        yield
+    elif cwd in sys.path:
         yield
     else:
         sys.path.insert(0, cwd)
@@ -69,7 +73,7 @@ def cwd_in_path():
 def find_module(module, path=None, imp=None):
     """Version of :func:`imp.find_module` supporting dots."""
     if imp is None:
-        imp = importlib.import_module
+        imp = import_module
     with cwd_in_path():
         try:
             return imp(module)
@@ -100,7 +104,7 @@ def import_from_cwd(module, imp=None, package=None):
     precedence over modules located in `sys.path`.
     """
     if imp is None:
-        imp = importlib.import_module
+        imp = import_module
     with cwd_in_path():
         return imp(module, package=package)
 
@@ -142,7 +146,14 @@ def gen_task_name(app, name, module_name):
 
 
 def load_extension_class_names(namespace):
-    for ep in entry_points().get(namespace, []):
+    if sys.version_info >= (3, 10):
+        _entry_points = entry_points(group=namespace)
+    else:
+        try:
+            _entry_points = entry_points().get(namespace, [])
+        except AttributeError:
+            _entry_points = entry_points().select(group=namespace)
+    for ep in _entry_points:
         yield ep.name, ep.value
 
 
