@@ -1,4 +1,5 @@
-from django.urls import handler404, handler500, include, url  # noqa
+from django.conf.urls import handler404, handler500  # noqa
+from django.urls import include, path  # noqa
 
 # Uncomment the next two lines to enable the admin:
 # from django.contrib import admin
