@@ -2,7 +2,7 @@ from celery import Celery
 
 app = Celery('proj',
              broker='amqp://',
-             backend='amqp://',
+             backend='rpc://',
              include=['proj.tasks'])
 
 # Optional configuration, see the application user guide.
