@@ -14,7 +14,7 @@ Usage::
 
    cd examples/security
 
-   (window1)$ python mysecureapp.py worker -l info
+   (window1)$ python mysecureapp.py worker -l INFO
 
    (window2)$ cd examples/security
    (window2)$ python
