@@ -1,5 +1,3 @@
-from __future__ import absolute_import, unicode_literals
-
 import operator
 
 import pytest
@@ -12,12 +10,18 @@ class test_thread_TaskPool:
     def test_on_apply(self):
         from celery.concurrency import thread
         x = thread.TaskPool()
-        x.on_apply(operator.add, (2, 2), {}, noop, noop)
+        try:
+            x.on_apply(operator.add, (2, 2), {}, noop, noop)
+        finally:
+            x.stop()
 
     def test_info(self):
         from celery.concurrency import thread
         x = thread.TaskPool()
-        assert x.info
+        try:
+            assert x.info
+        finally:
+            x.stop()
 
     def test_on_stop(self):
         from celery.concurrency import thread
